from lqs.interface.core.create import CreateInterface
from lqs.interface.core.delete import DeleteInterface
from lqs.interface.core.fetch import FetchInterface
from lqs.interface.core.list import ListInterface
from lqs.interface.core.update import UpdateInterface
